/**
 * This class combines several pads (given as URI's) into one
 * data stream, and gives it to HTTP client.
 */

package xorpads;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;


public class Combine extends UniformHttpServlet {
    /* Buffer Size for BufferedInputStreams */
    final private int BUFFER_SIZE = 64*1024;
    /* Size of a Chunk */
    final private int CHUNK_SIZE = 128*1024;
    /* Size of an output portion */
    final private int PORTION_SIZE = 4096;
    /* Proxy details */
    final private boolean USE_PROXY  = true;
    final private String  PROXY_HOST = "132.72.40.8";
    final private int     PROXY_PORT = 8080;
    
    /* GET request type */
    public void processRequest (HttpServletRequest request,
                                HttpServletResponse response)
        throws Exception {
        /* get the PrintWriter object */
        String type = request.getParameter("type");
        if(type == null) { type = "text/html"; }
        response.setContentType(type);
        PrintWriter out = response.getWriter();
        
        /* Determine the requested resource size */
        int remainder = Integer.parseInt(request.getParameter("size"));
        /* Initialize chunk number */
        int chunkNumber = 1;

        /* process all chunks */
        while(remainder > 0) {
            /* calculate current chunk size, and adjust remainder */
            int chunk = Math.min(remainder,CHUNK_SIZE);
            remainder -= chunk;

            /* get the input streams */
            DataInputStream[] streams = getInputStreams(request,chunkNumber);

            /* process all portions */
            while(chunk > 0) {
                /* calculate portion size, and adjust chunk */
                int portion = Math.min(chunk,PORTION_SIZE);
                chunk -= portion;

                /* get the data for this chunk */
                byte[] chunkData = XorPads.unpad(streams,portion);

                /* deliver data to the client */
                out.print(new String(chunkData,"ISO-8859-1"));
            }

            /* close the input streams */
            closeInputStreams(streams);
            
            /* proceed with next chunk number */
            chunkNumber++;
        }

        out.close();
    }

    /* Get DataInputStreams Array for chunk chunkNumber */
    private DataInputStream[] getInputStreams(HttpServletRequest request,
                                              int chunkNumber)
        throws MalformedURLException, IOException {
        /* get URLs for chunkNumber */
        String[] values = request.getParameterValues
            (String.valueOf(chunkNumber));
        /* construct InputStreams array */
        DataInputStream[] streams = new DataInputStream[values.length];
        /* build connections and extract InputStreams */
        for(int pad=0; pad<values.length; pad++) {
            URL url;
            if(USE_PROXY) {
                url = new URL("http",PROXY_HOST,PROXY_PORT,values[pad]);
            }
            else {
                url = new URL(values[pad]);
            }
            URLConnection con = url.openConnection();
            con.connect();
            streams[pad] = new DataInputStream
                (new BufferedInputStream
                    (con.getInputStream(),BUFFER_SIZE));
        }
        return streams;
    }

    /* Close all InputStreams */
    private void closeInputStreams(InputStream[] streams)
        throws IOException {
        for(int pad=0; pad<streams.length; pad++) {
            streams[pad].close();
        }
    }
}
