/**
 * Uniform HttpServlet interface, hides the difference
 * between GET and POST, and catches exceptions
 */

package xorpads;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

abstract public class UniformHttpServlet extends HttpServlet {
    /* GET request type */
    final public void doGet(HttpServletRequest request,
                            HttpServletResponse response)
        throws IOException {
        processCatchRequest(request,response);
    }

    /* POST request type */
    final public void doPost(HttpServletRequest request,
                             HttpServletResponse response)
        throws IOException {
        processCatchRequest(request,response);
    }

    /* overriding getServletInfo */
    public String getServletInfo() {
        return "(c) Michael Orlov, 2000, GNU General Public License";
    }

    /* Intermediate processing -- catching exceptions */
    public void processCatchRequest
        (HttpServletRequest request,HttpServletResponse response)
        throws IOException {
        try {
            processRequest(request,response);
        }
        catch(Exception e) {
            response.setContentType("text/html");
            PrintWriter out = response.getWriter();
            /* Show the stack trace */
            out.print("<html><head><title>ServletError</title></head>"
                      + "<body><hr /><h1>ServletError - "
                      + getServletContext().getServerInfo()
                      + "</h1><hr />"
                      + "<p>An error occured while processing your "
                      + "request:</p>"
                      + "<pre>");
            e.printStackTrace(out);
            out.print("</pre>");
            /* Show the request headers */
            out.print("<hr /><p>Here are the request headers:</p>");
            out.print("<table>");
            Enumeration headerNames = request.getHeaderNames();
            while(headerNames.hasMoreElements()) {
                String header = (String)headerNames.nextElement();
                out.print("<tr>");
                out.print("<td><em>"+header+"</em></td>");
                out.print("<td>"+request.getHeader(header)+"</td>");
                out.print("</tr>");
            }
            out.print("</table>");
            out.print("<hr /></body></html>");
        }
    }

    /* The following function must be implemented */
    abstract public void processRequest(HttpServletRequest request,
                                        HttpServletResponse response)
        throws Exception;
}
